{
  THKCurrEdit-Component

  Author: Harri Kasulke - Hamburg/Germany
  E-mail: harri.kasulke@okay.net
  Version: 1.0 (Developed and tested with D3 only)
  Date   : 21.12.98

  This component is freeware!
  Any comments or suggestions are welcome at: harri.kasulke@okay.net

  Disclaimer of warranty:
  This Component is supplied as is. The author disclaims all warranties, expressed
  or implied, including, without limitation, the warranties of merchantability
  and of fitness for any purpose. The author assumes no liability for damages,
  direct or consequential, which may result from the use of this Component.

  Component-Description:

  THKCurrEdit is a very "hard" masked Edit-Control with fixed Decimals
  and special Showing Mode (like 123.95 US$),
  developed for easy masked Currency-Editing.

  Special Properties:

   Decimals         - Sets the Decimals (0 to 4)
   Value            - Value (Type Currency)
   CurrencyStr      - Currency-Name for Show-mode (DEM, EUR, US$...)
   ThousandSep      - Thousandseparator-Showing On/Off (in Edit-Mode not shown)
   ExitWithEnter    - Performs a Tab-Key-Message with Return/Enter-Key (needful for editing multiple HKCurrEdits with NumPad)
   RestoreWithEsc   - Esc-Restore like DBEdit

  Special Public Methods:

   AsCurrency, AsFloat, AsString - Returns the Value as Currency, Float or String
   Update                        - Override for Updating the Value (needed for HKDBCurrEdit)


}

unit HKCEdit;

interface

uses
  WinTypes, WinProcs, Menus, Messages, SysUtils, Classes, Graphics,
  Controls, Forms, Dialogs, StdCtrls;

type
  HKCEditError=class(Exception);

  THKCurrEdit = class(TCustomEdit)
  private
    { Private-Deklarationen }
    FAlign: TAlignment;
    FValue: Currency;
    FMaxDigits: smallint;
    FDecimals: smallint;
    FCurrencyStr: string;
    FOldText: string;
    FShowKSeps: boolean;
    FExitWithEnter: boolean;
    FRestoreWithEsc: boolean;
    FEditMode: boolean;

    procedure SetTextFromValue;

    //Format- and String-Methods
    function FormatIt(Value: Currency; ShowMode: boolean): string;
    function LeftStr(OrgStr: string; CharCount: smallint): string;
    function RightStr(OrgStr: string; CharCount: smallint): string;
    function InsertChars(OrgStr, InsChars: string; CharPos : smallint): string;
    function DeleteChars(OrgStr: string; CharPos, CharCount: smallint): string;
    function ReplaceChars(OrgStr, ReplChars: string; CharPos : smallint): string;
    function ReplaceChars2(OrgStr: string; ReplChar: Char; CharPos, CharCount : smallint): string;

    //Property Get- and Set-Methods
    function GetDecimals: smallint;
    procedure SetDecimals(Value: smallint);
    function GetValue: Currency;
    procedure SetValue(Value: Currency);
    function GetCurrStr: string;
    procedure SetCurrStr(CurrStr: string);

  protected
    { Protected-Deklarationen }
    procedure KeyDown(var Key: Word; Shift: TShiftState); override;
    procedure KeyPress(var Key: Char); override;
    procedure DoEnter; override;
    procedure DoExit; override;

  public
    { Public-Deklarationen }
    constructor Create(AOwner: TComponent); override;
    procedure CreateParams(var Params: TCreateParams); override;
    procedure Update; override;

    function AsCurrency: Currency;
    function AsFloat: double;
    function AsString: string;

  published
    { Published-Deklarationen }
    property AutoSelect;
    property AutoSize;
    property BorderStyle;
    property Color;
    property Ctl3D;
    property DragCursor;
    property DragMode;
    property Enabled;
    property Font;
    property MaxLength;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ReadOnly;
    property ShowHint;
    property TabOrder;
    property TabStop;
    property Visible;
    property OnChange;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;

    //Extended properties
    property Decimals: smallint read GetDecimals write SetDecimals;
    property Value: Currency read GetValue write SetValue;
    property CurrencyStr: string read GetCurrStr write SetCurrStr;
    property ThousandSep: Boolean read FShowKSeps write FShowKSeps;
    property ExitWithEnter: Boolean read FExitWithEnter write FExitWithEnter;
    property RestoreWithEsc: Boolean read FRestoreWithEsc write FRestoreWithEsc;

  end;

procedure Register;

implementation

procedure Register;
begin
 RegisterComponents('HKComps', [THKCurrEdit]);
end;

constructor THKCurrEdit.Create(AOwner: TComponent);
begin
 inherited Create(AOwner);
 FCurrencyStr:='DM';           //Set Defaults
 FValue:=0;
 FDecimals:=2;
 FMaxDigits:=17-FDecimals;
 FAlign:=taRightJustify;
 FShowKSeps:=True;
 FExitWithEnter:=True;
 FRestoreWithEsc:=True;
 FEditMode:=False;
 SetTextFromValue;
end;

procedure THKCurrEdit.CreateParams(var Params: TCreateParams);
const Alignments: array[TAlignment] of Word = (ES_LEFT, ES_RIGHT, ES_CENTER);
begin
 inherited CreateParams(Params);
 Params.Style:=Params.Style or ES_NUMBER or ES_MULTILINE or Alignments[FAlign];
end;

procedure THKCurrEdit.Update;
begin
 inherited Update;
 SetTextFromValue;
end;

procedure THKCurrEdit.DoEnter;
begin
 FEditMode:=True;
 SetTextFromValue;
 FOldText:=Text;  //Backup the Text (for Undo with Esc)
 SelectAll;
 inherited DoEnter;
end;

procedure THKCurrEdit.DoExit;
begin
 FEditMode:=False;
 SetTextFromValue;
 inherited DoExit;
end;

procedure THKCurrEdit.KeyDown(var Key: Word; Shift: TShiftState);
var txt: string;
    tlen, cPos: integer;
    InputParsed: Boolean;
    DoBeep: Boolean;
begin
 if ReadOnly then Exit;

 DoBeep:=False;           //Defaults
 InputParsed:=False;

 if Key=VK_DELETE then                                   //del-button only
 begin
  cPos:=SelStart;
  txt:=FormatIt(StrToCurr(Text),False);
  tlen:=Length(txt);

  if ((cPos=tlen) or (cPos=tlen-FDecimals-1)) then       //forbidden Del-Positions
  begin
   InputParsed:=True;                                    //Job done
   DoBeep:=True;                                         //Input Error
  end;

  if ((not InputParsed) and (SelStart=0) and (SelLength=tlen)) then       //Delete complete
  begin
   txt:=FormatIt(0,False);
   InputParsed:=True;
  end;

  if ((not InputParsed) and (cPos>=tlen-FDecimals-1) and (cPos<tlen)) then     //Deleting in Decimal Area
  begin
   if SelLength=0 then
   begin
    if cPos<tlen-1 then txt:=DeleteChars(txt,cPos,1)+'0'     //Delete and Fillup with Zero
     else txt:=ReplaceChars(txt,'0',cPos);                   //Replace Zero on Last Position

   end
   else begin
    txt:=ReplaceChars2(txt,'0',cPos,SelLength);
   end;
   InputParsed:=True;                                        //Job done
  end;

  if ((not InputParsed) and(cPos<=tlen-FDecimals-1)) then     //General Deleting-Handling
  begin
   if SelLength=0 then
   begin
    if LeftStr(txt,1)<>'0' then                                 //No Leading Zero?
    begin
     if tlen=FDecimals+2 then txt:=ReplaceChars(txt,'0',cPos)  //Special-Handling for x.xx
      else txt:=DeleteChars(txt,cPos,1);
    end
    else begin
     DoBeep:=True;                                             //Cannot delete Leading Zero in this case!
    end;
   end
   else begin
    if cPos+SelLength<=tlen-FDecimals-1 then
    begin
     txt:=DeleteChars(txt,cPos,SelLength);
    end
    else begin
     DoBeep:=True;
    end;
   end;
  end;
  if Text<>txt then
  begin
   cPos:=SelStart;
   Text:=FormatIt(StrToCurr(txt),False);
   SelStart:=cPos;
   Modified:=True;
  end;
  Key:=0;                 //Eat the Key
 end;

 if DoBeep then Beep;

 inherited KeyDown(Key, Shift);

end;

procedure THKCurrEdit.KeyPress(var Key: Char);
var txt: string;
    tlen,cPos: integer;
    DelPressed: boolean;
    ExitFlag: boolean;
    Negative: boolean;
    InputParsed: boolean;
    DoBeep: boolean;
begin
 if ReadOnly then exit;

 InputParsed:=False;   //Defaults
 ExitFlag:=False;
 DoBeep:=False;

 if LeftStr(Text,1)='-' then Negative:=True     //Negative-Flag
  else Negative:=False;

 cPos:=SelStart;
 txt:=FormatIt(StrToCurr(Text),False);
 tlen:=Length(txt);

 if not(Key in['0'..'9', DecimalSeparator, '-', Char(VK_RETURN), Char(VK_ESCAPE), Char(#8)]) then      //Filter Keys
 begin
  DoBeep:=True;           //Wrong Key!
  InputParsed:=True;      //Job done
 end;

 if ((not InputParsed) and (Key=Char(#8))) then
 begin
  DelPressed:=True;    //Del-Pressed-Flag
  InputParsed:=True;
 end
 else begin
  DelPressed:=False;
 end;

 if ((not InputParsed) and (Key=Char(VK_RETURN))) then
 begin
  if FExitWithEnter then
  begin
   GetParentForm(Self).Perform(CM_DIALOGKEY, Byte(VK_TAB), 0);    //Return performs a Tab-Message for easy Numpad-Input
   ExitFlag:=True;
  end;
  InputParsed:=True;                           //Job done
 end;

 if ((not InputParsed) and (Key=Char(VK_ESCAPE))) then
 begin
  if FRestoreWithEsc then
  begin
   txt:=FOldText;       //Restore Backuped Text
   Modified:=False;     //Reset Modified-Flag
  end;
  InputParsed:=True;                           //Job done
 end;

 if (((not InputParsed)or (DelPressed)) and (SelStart=0) and (SelLength=tlen)) then       //Delete complete
 begin
  txt:=FormatIt(0,False);
  tlen:=Length(txt);
  cPos:=0;
  SelLength:=0;
 end;

 if ((not InputParsed) and (Key='-')) then       //Minus-Handling
 begin
  if ((cPos=0) and (not Negative)) then          //First Position and not Negative
  begin
   DoBeep:=False;
   if SelLength=0 then
   begin
    txt:=InsertChars(txt,'-',0);
    cPos:=cPos+1;                                //New Cursor-Pos
    Negative:=True;
   end
   else begin
    if cPos+SelLength<=tlen-FDecimals-1 then
    begin
     txt:=DeleteChars(txt,cPos,SelLength);       //Replace Selection with '-'
     txt:=InsertChars(txt,'-',0);
     cPos:=cPos+1;                               //New Cursor-Pos
     Negative:=True;
    end
    else DoBeep:=True;                           //Wrong Selection
   end;
  end
  else DoBeep:=True;                           //Wrong Position

  InputParsed:=True;                           //Job done
 end;

 if ((not InputParsed) and (Key='0')) then     //leading zero not wanted
 begin
  if ((cPos=0) and (not Negative)) then     //positive case
  begin
   DoBeep:=True;
   InputParsed:=True;                           //Job done
  end;

  if ((cPos<2) and (Negative)) then         //negative case
  begin
   DoBeep:=True;
   InputParsed:=True;                           //Job done
  end;

 end;

 if ((not InputParsed) and (FDecimals>0) and (Key=DecimalSeparator)) then //DecimalSeparator-Handling
 begin
  if cPos<tlen-FDecimals then
  begin
   cPos:=tlen-FDecimals;                    //Put Cursor to Decimals-Area
   DoBeep:=False;
  end
  else DoBeep:=True;                        //Wrong Position

  InputParsed:=True;                        //Job done
 end;

 if ((DelPressed) and (FDecimals>0) and (cPos>=tlen-FDecimals-1)) then  //Delete in Decimal Area
 begin
  if SelLength=0 then
  begin
   if cPos>tlen-FDecimals then
   begin
    txt:=ReplaceChars(txt,'0',cPos-1);            //Replace '0' instead of Deleting
    cPos:=cPos-1;                                 //New Cursor-Pos
    DoBeep:=False;
   end
   else DoBeep:=True;                            //Wrong Position
  end
  else begin
   txt:=ReplaceChars2(txt,'0',cPos,SelLength);
  end;
 end;

 if ((not InputParsed) and (FDecimals>0) and (cPos>=tlen)) then //Add no more Decimals at last Pos
 begin
  DoBeep:=True;                               //No more Decimals
  InputParsed:=True;                          //Job done
 end;

 if ((not InputParsed) and (FDecimals>0) and (cPos>=tlen-FDecimals)) then
 begin
  if SelLength=0 then
  begin
   txt:=ReplaceChars(txt,Key,cPos);             //Dont Insert, instead Overwrite Decimals in Decimal Area
   cPos:=cPos+1;                                //New Cursor-Pos
  end
  else begin
   if cpos>=tlen-FDecimals then
   begin
    txt:=ReplaceChars2(txt,'0',cPos,SelLength);
    txt:=ReplaceChars(txt,Key,cPos);            //Dont Insert, instead Overwrite Decimals in Decimal Area
    cPos:=cPos+1;                               //New Cursor-Pos
   end
   else DoBeep:=True;                           //Wrong Selection
  end;
  InputParsed:=True;                            //Job done
 end;

 if ((DelPressed) and (cPos<=tlen-FDecimals-1)) then  //General Delete-Handling
 begin
  if SelLength=0 then
  begin
   if cPos>0 then
   begin
    if tlen=FDecimals+2 then txt:=ReplaceChars(txt,'0',cPos-1)  //Special-Handling for x.xx
      else txt:=DeleteChars(txt,cPos-1,1);
    cPos:=cPos-1;                                //New Cursor-Pos
    DoBeep:=False;                               //Delete was Ok
   end
   else DoBeep:=True;                            //Wrong Position
  end
  else begin
   if cPos+SelLength<=tlen-FDecimals-1 then
   begin
    txt:=DeleteChars(txt,cPos,SelLength);
    SelStart:=cPos-1;
    DoBeep:=False;                               //Delete was Ok
   end
   else DoBeep:=True;                     //Wrong Selection
  end;
 end;

 if ((not InputParsed) and (cPos<=tlen-FDecimals-1)) then   //Input, Insert-Handling
 begin
  if SelLength=0 then
  begin
   if tlen<FMaxDigits then
   begin
    if (((LeftStr(txt,1)='0') and (cPos=0) and (not Negative)) or ((LeftStr(txt,2)='-0') and (cPos=1) and (Negative))) then
    begin
     txt:=ReplaceChars(txt,Key,cPos);            //Special Handling for -0.xx/0.xx
     cPos:=cPos+1;                               //New Cursor-Pos
     DoBeep:=False;
    end
    else begin
     if ((Negative) and (cPos=0)) then DoBeep:=True   //Wrong Position
     else begin
      txt:=InsertChars(txt,Key,cPos);
      cPos:=cPos+1;                             //New Cursor-Pos
      DoBeep:=False;
     end;
    end;
   end
   else DoBeep:=True;                           //No more Digits
  end
  else begin
   if cPos+SelLength<=tlen-FDecimals-1 then
   begin
    txt:=DeleteChars(txt,cPos,SelLength);
    txt:=InsertChars(txt,Key,cPos);
    DoBeep:=False;                               //Delete was Ok
   end
   else DoBeep:=True;                           //Wrong Selection
  end;
  InputParsed:=True;                            //Job done
 end;   

 if ((not ExitFlag) and (InputParsed)) then
 begin
  if Text<>txt then
  begin
   Text:=FormatIt(StrToCurr(txt),False);   //Reformat
   if ((Negative) and (LeftStr(Text,1)<>'-')) then Text:=InsertChars(Text,'-',0);
   Modified:=True;
  end;
  SelStart:=cPos;
  key:=#0;
 end;

 if DoBeep then Beep;

 inherited KeyPress(Key);
 
end;

function THKCurrEdit.GetDecimals: smallint;
begin
 result:=FDecimals;
end;

procedure THKCurrEdit.SetDecimals(Value: smallint);
begin
 if ((Value>=0) and(Value<=4)) then
 begin
  FDecimals:=Value;
 end
 else begin
  FDecimals:=2;
  raise HKCEditError.Create('"'+IntToStr(Value)+'" is not valid for Decimals (0 to 4)');
 end;
 Update;
end;

function THKCurrEdit.GetValue: Currency;
begin
 result:=FValue;
end;

procedure THKCurrEdit.SetValue(Value: Currency);
var test: string;
begin
 try
  test:=FormatCurr('0.00',Value);
 except
  FValue:=0;
  raise HKCEditError.Create('"'+FloatToStr(Value)+'" is not valid for Value');
 end;
 FValue:=Value;
 Update;
end;

function THKCurrEdit.GetCurrStr: string;
begin
 result:=FCurrencyStr;
end;

procedure THKCurrEdit.SetCurrStr(CurrStr: string);
begin
 FCurrencyStr:=CurrStr;
 Update;
end;

procedure THKCurrEdit.SetTextFromValue;
begin
 if FEditMode then Text:=FormatIt(FValue, False)
  else Text:=FormatIt(FValue,True)+' '+FCurrencyStr;
end;

//Special Public-Methods

function THKCurrEdit.AsCurrency: Currency;
begin
 result:=FValue;
end;

function THKCurrEdit.AsFloat: double;
begin
 result:=FValue;
end;

function THKCurrEdit.AsString: string;
begin
 result:=CurrToStr(FValue);
end;

// Private Format- and String-Methods
function THKCurrEdit.FormatIt(Value: Currency; ShowMode: boolean): string;
var decimals, mask: string;
    x : smallint;
begin
 decimals:='';
 if ShowMode and FShowKSeps then mask:=',0.'
   else mask:='0.';
 for x:=0 to FDecimals-1 do
  decimals:=decimals+'0';
 mask:=mask+decimals;
 try
  result:=FormatCurr(mask, Value);
  if not ShowMode then FValue:=StrToCurr(result);; //Update Value
 except
  result:='0'+DecimalSeparator+decimals;
  if not ShowMode then
  begin
   FValue:=0;
   SetTextFromValue;
  end;
  raise HKCEditError.Create('"'+FloatToStr(Value)+'" is not a valid value');
 end;
end;

function THKCurrEdit.LeftStr(OrgStr: string; CharCount: smallint): string;
begin
 try
  result:=Copy(OrgStr,0,CharCount);
 except
  result:='';
 end;
end;

function THKCurrEdit.RightStr(OrgStr: string; CharCount: smallint): string;
begin
 result:=Copy(OrgStr,(Length(OrgStr)-CharCount)+1,Length(OrgStr));
end;

function THKCurrEdit.DeleteChars(OrgStr: string; CharPos, CharCount: smallint): string;
begin
 result:=LeftStr(OrgStr,CharPos)+RightStr(OrgStr,Length(OrgStr)-CharPos-CharCount);
end;

function THKCurrEdit.InsertChars(OrgStr, InsChars: string; CharPos : smallint): string;
begin
 result:=LeftStr(OrgStr,CharPos)+InsChars+RightStr(OrgStr,Length(OrgStr)-CharPos);
end;

function THKCurrEdit.ReplaceChars(OrgStr, ReplChars: string; CharPos : smallint): string;
begin
 result:=LeftStr(OrgStr,CharPos)+ReplChars+RightStr(OrgStr,Length(OrgStr)-CharPos-Length(ReplChars));;
end;

function THKCurrEdit.ReplaceChars2(OrgStr: string; ReplChar: Char; CharPos, CharCount : smallint): string;
var x: smallint;
begin
 result:=OrgStr;
 for x:=0 to CharCount-1 do
 begin
  result:=ReplaceChars(result,ReplChar,CharPos+x);
 end;
end;

end.
