{
  THKDBCurrEdit-Component  (DB-Version of THKCurrEdit)

  Author: Harri Kasulke - Hamburg/Germany
  E-mail: harri.kasulke@okay.net
  Version: 1.0 (Developed and tested with D3 only)
  Date   : 21.12.98

  This component is freeware!
  Any comments or suggestions are welcome at: harri.kasulke@okay.net

  Disclaimer of warranty:
  This Component is supplied as is. The author disclaims all warranties, expressed
  or implied, including, without limitation, the warranties of merchantability
  and of fitness for any purpose. The author assumes no liability for damages,
  direct or consequential, which may result from the use of this Component.

  For Component-Description see THKCEdit.pas

}

unit HKDBCEdit;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Dbctrls, Db, HKCEdit;

type
  HKDBCEditError=class(Exception);

  THKDBCurrEdit = class(THKCurrEdit)
  private
    { Private-Deklarationen }
    FDataLink: TFieldDataLink;
    function GetDataField: string;
    function GetDataSource: TDataSource;
    function GetField: TField;
    procedure SetDataField(const Value: string);
    procedure SetDataSource(Value: TDataSource);
    procedure CMGetDataLink(var Message: TMessage); message CM_GETDATALINK;

  protected
    { Protected-Deklarationen }
    procedure DataChange(Sender: TObject);
    procedure UpdateData(Sender: TObject);
    procedure ActiveChanged(Sender: TObject);
    procedure Change; override;
    procedure KeyPress(var Key: Char); override;
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
  public
    { Public-Deklarationen }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    property Field: TField read GetField;

  published
    { Published-Deklarationen }
    property DataField: string read GetDataField write SetDataField;
    property DataSource: TDataSource read GetDataSource write SetDataSource;
    property AutoSelect;
    property AutoSize;
    property BorderStyle;
    property Color;
    property Ctl3D;
    property DragCursor;
    property DragMode;
    property Enabled;
    property Font;
    property MaxLength;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ReadOnly;
    property ShowHint;
    property TabOrder;
    property TabStop;
    property Visible;
    property OnChange;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;

    //Special HKCEdit properties
    property Value;
    property CurrencyStr;
    property ThousandSep;
    property ExitWithEnter;

  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('HKComps', [THKDBCurrEdit]);
end;

constructor THKDBCurrEdit.Create(AOwner: TComponent);
begin
 inherited Create(AOwner);
 FDataLink:=TFieldDataLink.Create;
 FDataLink.Control:=Self;
 FDataLink.OnDataChange:=DataChange;
 FDataLink.OnUpdateData:=UpdateData;
 FDataLink.OnActiveChange:=ActiveChanged;
end;

destructor THKDBCurrEdit.Destroy;
begin
 FDataLink.Free;
 FDataLink:=nil;
 inherited Destroy;
end;

procedure THKDBCurrEdit.Notification(AComponent: TComponent; Operation: TOperation);
begin
 inherited Notification(AComponent, Operation);
 if (Operation = opRemove) and (FDataLink <> nil) and (AComponent = DataSource) then DataSource:=nil;
end;

procedure THKDBCurrEdit.KeyPress(var Key: Char);
begin
 if Key=Char(VK_ESCAPE) then                    //Special-Handling for Esc-Key
 begin
  FDataLink.Reset;                              //Restore Field-Data
  Key:=#0;                                      //Eat the Key
 end
 else FDataLink.Edit;
 inherited KeyPress(Key);
end;

procedure THKDBCurrEdit.Change;
begin
 inherited Change;
 if Modified then FDataLink.Modified;
end;

procedure THKDBCurrEdit.CMGetDatalink(var Message: TMessage);
begin
 Message.Result:=Integer(FDataLink);
end;

function THKDBCurrEdit.GetDataSource: TDataSource;
begin
 Result:=FDataLink.DataSource;
end;

procedure THKDBCurrEdit.SetDataSource(Value: TDataSource);
begin
 FDataLink.DataSource:=Value;
 if Value<>nil then Value.FreeNotification(Self);
end;

function THKDBCurrEdit.GetDataField: string;
begin
 Result:=FDataLink.FieldName;
end;

procedure THKDBCurrEdit.SetDataField(const Value: string);
begin
 FDataLink.FieldName:=Value;
end;

function THKDBCurrEdit.GetField: TField;
begin
 Result:=FDataLink.Field;
end;

procedure THKDBCurrEdit.DataChange(Sender: TObject);
var FTypeOK : boolean;
begin
 FTypeOK:=False;
 if FDataLink.Field<>nil then
 begin                              //TField
  try
   if FDataLink.Field is TFloatField then
   begin
    Value:=FDataLink.Field.AsFloat;
    FTypeOK:=True;
   end;

   if FDataLink.Field is TCurrencyField then
   begin
    Value:=FDataLink.Field.AsFloat;
    FTypeOK:=True;
   end;

   if FDataLink.Field is TStringField then
   begin
    if FDataLink.Field.AsString='' then Value:=0
     else Value:=FDataLink.Field.AsFloat;
    FTypeOK:=True;
   end;

   if not FTypeOK then
   begin
    Value:=0;
    raise HKDBCEditError.Create('Fieldtype of Field "'+FDataLink.Field.FieldName+'" is not supported');
   end;

   Modified:=False;
   Update;
   
  except
   raise HKDBCEditError.Create('Cannot convert Field "'+FDataLink.Field.FieldName+'" to Float');
  end;
 end;
end;

procedure THKDBCurrEdit.UpdateData(Sender: TObject);
var FTypeOK : boolean;
begin
 FTypeOK:=False;
 try
  if FDataLink.Field is TFloatField then
  begin
   FDataLink.Field.AsFloat:=Value;
   FTypeOK:=True;
  end;

  if FDataLink.Field is TCurrencyField then
  begin
   FDataLink.Field.AsCurrency:=Value;
   FTypeOK:=True;
  end;

  if FDataLink.Field is TStringField then
  begin
   FDataLink.Field.AsString:=CurrToStr(Value);
   FTypeOK:=True;
  end;

  if not FTypeOK then
  begin
   raise HKDBCEditError.Create('Fieldtype of Field "'+FDataLink.Field.FieldName+'" is not supported');
  end;

 except
  raise HKDBCEditError.Create('Cannot convert Data for Field "'+FDataLink.Field.FieldName+'"');
 end;
end;

procedure THKDBCurrEdit.ActiveChanged(Sender: TObject);
begin
 if FDataLink.Active then
 begin
  DataChange(nil);
 end;
end;

end.
